<?php

/**
 * Custom renderer for Glopal status in System Configuration
 *
 * @category   Glopal
 * @package    Glopal_Connector
 * @author     Glopal Team <support@glopal.com>
 * @license    http://opensource.org/licenses/osl-3.0.php
 */
class Glopal_Connector_Block_Adminhtml_System_Config_Fieldset_Settings extends Mage_Adminhtml_Block_System_Config_Form_Fieldset
{
    /**
     * Calls APi to render configuration JS
     */
    public function render(Varien_Data_Form_Element_Abstract $element)
    {
        $storeId = $this->_getStoreId();

        $notified = $storeId && Mage::helper('glopal_connector/instalation')->isStoreInstallationNotified( $storeId );
        
        if( $storeId && ! $notified ) {
            Mage::helper('glopal_connector/instalation')->notifyInstalation( true );
            $notified = Mage::helper('glopal_connector/instalation')->isStoreInstallationNotified( $storeId );
        }

        $this->_renderExternalUi($storeId, $notified);
    }

    private function _renderExternalUi( $storeId, $notified )
    {
        try {

            $data = $this->_getExternalUiData( $storeId, $notified );
            $config = $this->_buildExternalUiConfig( $data, $storeId );
            $src = $data['script_src'];
            $this->_renderHtml( $src, $config );
        
        } catch (Exception $e) {
            Mage::getConfig()->saveConfig( 'webinterpret_connector/last_request_error_load', $e->getMessage() . "\n" . $e->getTraceAsString(), 'stores', $storeId );
            Mage::getConfig()->reinit();
            $this->_renderError($e->getCode());
        }
    }

    /**
     * Create configuration for external js
     */
    private function _buildExternalUiConfig( $data, $storeId )
    {
        $config = $data['config'];
        $config['magento_adminview_api_urls'] = array(
            'config' => $this->getUrl('adminhtml/glopal/config'),
            'stores' => $this->getUrl('adminhtml/glopal/stores'),
        );
        $config['magento_config'] = array();

        if( isset( $data['prefetch'] ) && is_array( $data['prefetch'] ) ) {
            foreach( $data['prefetch'] as $key ) {

                if( $key[0] == ':' ) {
                    $config['magento_config'][$key] = $this->_prefetchGlopal( $key );
                } else {
                    $config['magento_config'][$key] = Mage::getStoreConfig( $key, $storeId );
                }
            }
        }

        return $config;
    }

    /**
     * prefetch _webinterpret keys
     */
    private function _prefetchGlopal( $key )
    {
        switch( $key ) {
            case ':user:firstname' : $value = Mage::getSingleton('admin/session')->getUser()->getFirstname();
                break;
            case ':user:lastname' : $value = Mage::getSingleton('admin/session')->getUser()->getLastname();
                break;
            case ':user:email' : $value = Mage::getSingleton('admin/session')->getUser()->getEmail();
                break;
            default : $value = '';
        }

        return $value;
    }

    /**
     * Get WebInterpret configuration
     */
    private function _getExternalUiData( $storeId, $notified )
    {
        $request = new Varien_Object();
        $request->setStoreViewId( $storeId ? $storeId : null );
        $request->setMagentoUserEmail( Mage::getSingleton('admin/session')->getUser()->getEmail() );
        $request->setMagentoVersion( Mage::getVersion() );
        $request->setMagentoPluginVersion( Mage::helper('glopal_connector')->getExtensionVersion() );
        $request->setIsInstallationNotified( $notified );
        $request->setStoreKey( Mage::helper('glopal_connector')->getGlopalKey() );
        $request->setShortName( Mage::getStoreConfig( 'webinterpret_connector/short_name', $storeId ) );

        $data = Mage::helper('core')->jsonEncode($request);
        $url = Mage::getStoreConfig('webinterpret_connector/url_load_magento');

        $response = Mage::helper('glopal_connector')->sendPostRequest($url, $data);

        if(( $json = json_decode( $response, true ) ) === NULL) {
            throw new \Exception("Invalid JSON response: {$response}");
        }

        if( ! isset( $json['script_src'] ) || ! isset( $json['config'] ) ) {
            throw new \Exception("Invalid response schema: {$response}");
        }

        return $json;
    }

    /**
     * Get the current or default store ID
     */
    private function _getStoreId()
    {
        $storeId = 0;

        if( in_array( Mage::getSingleton('adminhtml/config_data')->getScope(), array( 'websites', 'default' ) ) ) {

            // Load the default store if any
            $stores = Mage::app()->getStores();
            if( count( $stores ) == 1 ) {
                $store = reset( $stores );
                $storeId = $store->getStoreId();
            }

        } else {
            $storeId = Mage::getSingleton('adminhtml/config_data')->getScopeId();
        }

        // TODO one more chance to grab the store id
        if (empty($storeId)) {
            $store = Mage::app()->getRequest()->getParam('store');
            $storeId = Mage::getModel('core/store')->load($store)->getId();
        }

        return $storeId;
    }

    /**
     * Output HTML for config API
     */
    private function _renderHtml( $src, $config ) {
        ?>
        <div id="webinterpret-placeholder" class="webinterpret-placeholder"><?php echo __('Loading...') ?></div>
        <script>
        window.webinterpretReady = function() {
            webinterpret.render("webinterpret-placeholder", <?=json_encode($config)?>)
        }
        </script>
        <script crossorigin="anonymous" src="<?=$src?>" async></script>
        <?php
    }

    /**
     * Render error HTML
     *
     * @param int $code
     */
    private function _renderError( $code = 0 )
    {
        ?>
            <ul class="messages"><li class="error-msg"><ul><li><span><?php echo __(
                    $code == 401 ? 'Failed to authenticate when loading Glopal extension' : 'Failed to load Glopal extension'
            ) ?></span></li></ul></li></ul>
        <?php
    }
}
