<?php

/**
 * Instalation helper
 *
 * @category   Glopal
 * @package    Glopal_Connector
 * @author     Glopal Team <support@glopal.com>
 * @license    http://opensource.org/licenses/osl-3.0.php
 */

class Glopal_Connector_Helper_Instalation extends Glopal_Connector_Helper_Data
{

    /**
     * Check if store view is registered
     */
    public function isStoreInstallationNotified($storeId) {
        return strlen( Mage::getStoreConfig('webinterpret_connector/installation_notified_at', $storeId) ) > 0;
    }

    /**
     * Notify WI about the store view
     */
    public function notifyInstalation( $force=false ) {

        if( ! $force ) {
            $retry = Mage::getStoreConfig('webinterpret_connector/installation_notify_retry');

            if( ! empty( $retry ) && ( new DateTime() < new DateTime($retry) ) ) {
                // scheduled for future attempt
                return;
            }
        }

        $url = Mage::getStoreConfig('webinterpret_connector/url_install_magento');
        $requestBody = $this->getStoresData();
        $data = Mage::helper('core')->jsonEncode( $requestBody );

        try {

            $response = Mage::helper('glopal_connector')->sendPostRequest($url, $data);

            if(( $json = json_decode( $response, true ) ) === NULL) {
                throw new \Exception("Invalid JSON response: {$response}");
            }

            if( isset($json['magento_config']) && is_array($json['magento_config']) && count($json['magento_config'])) {
                foreach( $json['magento_config'] as $data ) {
                    Mage::getConfig()->saveConfig( $data['path'], $data['value'], $data['scope'], $data['scope_id'] );
                }
                Mage::getConfig()->reinit();
            }

            return true;
        
        } catch (Exception $e) {
            Mage::getConfig()->saveConfig( 'webinterpret_connector/last_request_error_notify', $e->getMessage() . "\n" . $e->getTraceAsString());
            Mage::getConfig()->reinit();
        }

        $this->_scheduleNextNotify();

        return false;
    }

    private function _scheduleNextNotify()
    {
        $newDate = new DateTime();
        $newDate->modify("+5 minutes");
        $stamp = $newDate->format('Y-m-d H:i:s');
        Mage::getConfig()->saveConfig( 'webinterpret_connector/installation_notify_retry', $stamp );
        Mage::getConfig()->reinit();
    }

    /**
     * Get store base url based on secure settings
     */
    public function getStoreBaseUrl($storeId=0)
    {
        if( Mage::getStoreConfig('web/secure/use_in_frontend', $storeId) ) {
            return Mage::getStoreConfig('web/secure/base_url', $storeId);
        }
        return Mage::getStoreConfig('web/unsecure/base_url', $storeId);
    }

    /**
     * Get all website / stores data
     */
    public function getStoresData()
    {
        $data = array(
            'plugin_version' => $this->getExtensionVersion(),
            'magento_version' => Mage::getVersion(),
            'webinterpret_key' => $this->getGlopalKey(),
            'store_key' => $this->getGlopalKey(),
            'websites' => array(),
        );

        $websites = Mage::app()->getWebsites();

        foreach( $websites as $website ) {

            $websiteData = array(
                'id' => $website->getWebsiteId(),
                'code' => $website->getCode(),
                'name' => $website->getName(),
                'is_default' => $website->getIsDefault(),
                'stores' => array(),
            );
            $defaultGroup = $website->getDefaultGroupId();
            $groups = $website->getGroups();

            foreach( $groups as $group ) {

                $groupData = array(
                    'id' => $group->getGroupId(),
                    'name' => $group->getName(),
                    'is_default' => $defaultGroup == $group->getGroupId(),
                    'store_views' => array(),
                );
                $defaultStore = $group->getDefaultStoreId();
                $stores = $group->getStores();

                foreach( $stores as $store ) {

                    $storeData = array(
                        'id' => $store->getStoreId(),
                        'website_id' => $website->getWebsiteId(),
                        'group_id' => $group->getGroupId(),
                        'code' => $store->getCode(),
                        'name' => $store->getName(),
                        'is_default' => $defaultStore == $store->getStoreId(),
                        'is_active' => $store->getIsActive(),
                        'url' => $this->getStoreBaseUrl( $store->getStoreId() ),
                        'country' => Mage::getStoreConfig( 'general/store_information/merchant_country', $store->getStoreId() ),
                        'shipping_origin' => Mage::getStoreConfig( 'shipping/origin/country_id', $store->getStoreId() ),
                        'locale' => Mage::getStoreConfig( 'general/locale/code', $store->getStoreId() ),
                        'currency' => Mage::getStoreConfig( 'currency/options/base', $store->getStoreId() ),
                        'store_name' => Mage::getStoreConfig( 'general/store_information/name', $store->getStoreId() ),
                        'store_contact_email' => Mage::getStoreConfig( 'trans_email/ident_general/email', $store->getStoreId() ),
                        'store_contact_phone' => Mage::getStoreConfig( 'general/store_information/phone', $store->getStoreId() ),
                        'wi_installation_notified_at' => Mage::getStoreConfig( 'webinterpret_connector/installation_notified_at', $store->getStoreId() ),
                        'wi_short_name' => Mage::getStoreConfig( 'webinterpret_connector/short_name', $store->getStoreId() ),
                    );
                    $groupData['store_views'][]= $storeData;
                }

                $websiteData['stores'][] = $groupData;
            }

            $data['websites'][]= $websiteData;
        }

        return $data;
    }
}
