<?php
/**
 * Oder Cron
 *
 * @category   Glopal
 * @package    Glopal_Connector
 * @author     Glopal Team <support@glopal.com>
 * @license    http://opensource.org/licenses/osl-3.0.php
 */
class Glopal_Connector_Model_Cron_Order
{
    const ORDER_READY = 'order.ready';
    const ORDER_CANCELLED = 'order.cancelled';
    const ORDER_RECEIVED = 'order.received';

    const EVENTS_URL_PATTERN = 'v2/events/%s?event=%s&event=%s&event=%s&after_event_id=%s';
    const GET_ORDER_URL_PATTERN = 'v2/orders/%s/%s';
    const ACCEPT_ORDER_URL_PATTERN = 'v2/orders/%s/%s/accept';

    /** @var Glopal_Connector_Helper_Api */
    private $apiHelper;

    /** @var string|null*/
    private $urlApiFacade;

    /** @var array */
    private $lastSeenEventId;

    /** @var array|null */
    private $loadedOrdersData;

    public function __construct()
    {
        $this->apiHelper = Mage::helper('glopal_connector/api');
        $this->urlApiFacade = Mage::getStoreConfig('webinterpret_connector/url_api_facade');

        $lastSeenEventIdString = Mage::getStoreConfig('glopal_checkout/general/after_event_id');
        if (empty($lastSeenEventIdString)) {
            $this->lastSeenEventId = [];
        } else {
            $this->lastSeenEventId = Mage::helper('core')->jsonDecode($lastSeenEventIdString);
        }
    }

    /**
     * Process Glopal orders.
     *
     * @return $this
     * @throws Mage_Core_Exception
     */
    public function process()
    {
        $stores = Mage::app()->getStores();
        $shortNames = array();
        /** @var Mage_Core_Model_Store $store */
        foreach ($stores as $store) {
            $storeId = $store->getId();
            if (!$this->apiHelper->isAvailable($storeId)) {
                continue;
            }

            $shortName = Mage::getStoreConfig('webinterpret_connector/short_name', $storeId);
            if (!isset($shortNames[$shortName])) {
                // pick the first api key for the short name
                $apiKey = Mage::getStoreConfig('glopal_checkout/general/glopal_api_key', $storeId);
                $shortNames[$shortName] = $apiKey;
            }
        }

        foreach ($shortNames as $shortName => $apiKey) {
            $events = $this->getEvents($shortName, $apiKey);
            if (!count($events)) {
                $this->echoToCli(array(array('color' => '[32m', 'msg' => 'No new events found')));
            }

            foreach ($events as $event) {
                try {
                    if ($event['event_name'] == self::ORDER_RECEIVED) {
                        $this->processReceived($event, $shortName, $apiKey);
                    } else if ($event['event_name'] == self::ORDER_READY) {
                        $this->processReady($event, $shortName, $apiKey);
                    } else if ($event['event_name'] == self::ORDER_CANCELLED) {
                        $this->processCancelled($event, $shortName, $apiKey);
                    }

                    $this->lastSeenEventId[$shortName] = $event['event_id'];
                    Mage::getConfig()->saveConfig(
                        'glopal_checkout/general/after_event_id',
                        Mage::helper('core')->jsonEncode($this->lastSeenEventId)
                    );
                    $this->echoToCli(
                        array(
                            array(
                                'color' => '[32m',
                                'msg' => 'Saving last seen event id: '
                                    . $this->lastSeenEventId[$shortName]
                                    . ' for short name: '
                                    . $shortName
                            )
                        )
                    );
                } catch (Exception $e) {
                    Mage::logException($e);
                    $this->echoToCli(array(array('color' => '[31m', 'msg' => $e->getMessage())));

                    throw $e;
                } finally {
                    Mage::getConfig()->reinit();
                }
            }
        }

        return $this;
    }

    /**
     * Canceling suspended Magento orders.
     *
     * @return $this
     * @throws Mage_Core_Exception|Exception
     */
    public function processCancel()
    {
        $this->echoToCli(
            array(
                array('color' => '[32m', 'msg' => 'Start canceling suspended orders.')
            )
        );
        /** @var Mage_Sales_Model_Resource_Order_Collection $ordersCollection */
        $ordersCollection = Mage::getModel('sales/order')->getCollection();
        $ordersCollection->addAttributeToFilter(
            'status',
            array('eq' => Glopal_Connector_Model_Payment_Glopal::ORDER_GLOPAL_STATUS)
        )->addAttributeToFilter(
            'state',
            array('eq' => Mage_Sales_Model_Order::STATE_NEW)
        )->addFieldToFilter(
            'glopal_order_id',
            array('null' => true)
        )->addAttributeToFilter(
            'created_at',
            array('lt' => Mage::getModel('core/date')->date('Y-m-d H:i:s', strtotime('-1 day')))
        );

        $this->echoToCli(
            array(
                array('color' => '[33m', 'msg' => 'Orders to cancel count: ' . count($ordersCollection->getItems()))
            )
        );

        /** @var Mage_Sales_Model_Order $order */
        foreach ($ordersCollection as $order) {
            $order->addStatusHistoryComment(
                Mage::helper('glopal_connector')->__('Customer didn\'t finish the payment process.')
            );
            $order->cancel()->save();
        }

        return $this;
    }

    /**
     * Getting Glopal Received/Ready/Cancelled events stream.
     *
     * @param string $shortName
     * @param string $apiKey
     *
     * @return array
     * @throws Exception
     */
    private function getEvents($shortName, $apiKey)
    {
        if (isset($this->lastSeenEventId[$shortName])) {
            $lastSeenEventId = $this->lastSeenEventId[$shortName];
        } else {
            $lastSeenEventId = 1;
        }

        $url = sprintf(
            $this->urlApiFacade . self::EVENTS_URL_PATTERN,
            $shortName,
            self::ORDER_RECEIVED,
            self::ORDER_READY,
            self::ORDER_CANCELLED,
            $lastSeenEventId
        );

        $this->echoToCli(array(array('color' => '[32m', 'msg' => 'Getting events stream:')));
        $responseData = $this->apiHelper->makeGetHttpCall($url, $apiKey);
        $this->echoToCli(
            array(
                array('color' => '[33m', 'msg' => 'Events Response:'),
                array('color' => '[0m', 'msg' => Mage::helper('core')->jsonEncode($responseData))
            )
        );

        return $responseData['events'];
    }

    /**
     * Processing received orders.
     *
     * @param array $event
     * @param string $shortName
     * @param string $apiKey
     *
     * @return void
     * @throws Mage_Core_Exception
     * @throws Exception
     */
    private function processReceived($event, $shortName, $apiKey)
    {
        $glopalOrderId = $event['order_id'];
        $url = sprintf(
            $this->urlApiFacade . self::GET_ORDER_URL_PATTERN,
            $shortName,
            $glopalOrderId
        );

        // Load, set glopal order id, cache the order.
        $orderData = $this->getOrderData($url, $glopalOrderId, $apiKey);
        $magentoOrder = $orderData['magento_order'];
        $magentoOrder->setData('glopal_order_id', $glopalOrderId);

        $this->saveOrderData($glopalOrderId, $magentoOrder, $orderData);
    }

    /**
     * Processing ready orders.
     *
     * @param array $event
     * @param string $shortName
     * @param string $apiKey
     *
     * @return void
     * @throws Mage_Core_Exception
     */
    private function processReady($event, $shortName, $apiKey)
    {
        $glopalOrderId = $event['order_id'];
        $url = sprintf(
            $this->urlApiFacade . self::GET_ORDER_URL_PATTERN,
            $shortName,
            $glopalOrderId
        );

        $orderData = $this->getOrderData($url, $glopalOrderId, $apiKey);
        /** @var Mage_Sales_Model_Order $magentoOrder */
        $magentoOrder = $orderData['magento_order'];
        $glopalOrderTotal = (float)$orderData['glopal_order_total'];

        if ($magentoOrder->getData('glopal_order_id')
            && $magentoOrder->getStatus() !== Glopal_Connector_Model_Payment_Glopal::ORDER_GLOPAL_STATUS
        ) {
            $this->echoToCli(
                array(
                    array('color' => '[32m', 'msg' => 'Order with Glopal ID ' . $glopalOrderId . ' already accepted'),
                )
            );
            return;
        } else {
            $magentoOrder->setData('glopal_order_id', $glopalOrderId);

            /** @var Glopal_Connector_Model_Payment_Glopal $paymentMethodInstance */
            $paymentMethodInstance = $magentoOrder->getPayment()->getMethodInstance();
            $isGenerateInvoice = (bool)$paymentMethodInstance->getConfigData(
                'generate_invoice',
                $magentoOrder->getStoreId()
            );

            // Generate invoice if enabled and amount in Glopal order is matching amount in Magento order
            if ($isGenerateInvoice && $glopalOrderTotal == $magentoOrder->getGrandTotal()) {
                $this->generateInvoice($magentoOrder);
                $this->echoToCli(
                    array(
                        array('color' => '[32m', 'msg' => 'Generating an invoice.'),
                    )
                );
            } else {
                $magentoOrder->setBaseTotalPaid($glopalOrderTotal);
                $magentoOrder->setTotalPaid($glopalOrderTotal);
            }
        }

        $message = Mage::helper('glopal_connector')->__('Payment of %s %s completed with Glopal.');
        $magentoOrder->setState(
            Mage_Sales_Model_Order::STATE_PROCESSING,
            Mage_Sales_Model_Order::STATE_PROCESSING,
            sprintf($message, $magentoOrder->getOrderCurrencyCode(), $glopalOrderTotal)
        );

        $this->echoToCli(
            array(
                array('color' => '[32m', 'msg' => 'Saving the order with status:'),
                array('color' => '[33m', 'msg' => $magentoOrder->getStatus()),
            )
        );

        $this->saveOrderData($glopalOrderId, $magentoOrder, $orderData);

        // Accepting the order on Glopal
        $this->acceptOrder($shortName, $apiKey, $glopalOrderId, $magentoOrder);
    }

    /**
     * Generating invoice for an given order.
     *
     * @param Mage_Sales_Model_Order $order
     *
     * @return void
     */
    private function generateInvoice(Mage_Sales_Model_Order $order)
    {
        if ($order->canInvoice()) {
            /** @var Mage_Sales_Model_Order_Invoice $invoice */
            $invoice = Mage::getModel('sales/service_order', $order)->prepareInvoice();
            $invoice->setRequestedCaptureCase(Mage_Sales_Model_Order_Invoice::CAPTURE_ONLINE);
            $invoice->register();
            $invoice->getOrder()->setCustomerNoteNotify(false);
            $invoice->getOrder()->setIsInProcess(true);
            $transactionSave = Mage::getModel('core/resource_transaction')
                ->addObject($invoice)
                ->addObject($invoice->getOrder());
            $transactionSave->save();

            $order->setTotalPaid($order->getGrandTotal());
        }
    }

    /**
     * Processing ready orders.
     *
     * @param array $event
     * @param string $shortName
     * @param string $apiKey
     *
     * @return void
     * @throws Mage_Core_Exception
     */
    private function processCancelled($event, $shortName, $apiKey)
    {
        $glopalOrderId = $event['order_id'];
        $url = sprintf(
            $this->urlApiFacade . self::GET_ORDER_URL_PATTERN,
            $shortName,
            $glopalOrderId
        );

        $orderData = $this->getOrderData($url, $glopalOrderId, $apiKey);
        $magentoOrder = $orderData['magento_order'];

        if ($magentoOrder->getData('glopal_order_id')
            && $magentoOrder->getStatus() !== Glopal_Connector_Model_Payment_Glopal::ORDER_GLOPAL_STATUS
        ) {
            $this->echoToCli(
                array(
                    array('color' => '[32m', 'msg' => 'Order with Glopal ID ' . $glopalOrderId . ' already accepted'),
                )
            );
            return;
        } else {
            $magentoOrder->setData('glopal_order_id', $glopalOrderId);
        }

        $magentoOrder->setState(
            Mage_Sales_Model_Order::STATE_CANCELED,
            Mage_Sales_Model_Order::STATE_CANCELED,
            Mage::helper('glopal_connector')->__('Order was canceled on Glopal side.')
        );

        $this->echoToCli(
            array(
                array('color' => '[32m', 'msg' => 'Saving the order with status:'),
                array('color' => '[33m', 'msg' => $magentoOrder->getStatus()),
            )
        );

        $this->saveOrderData($glopalOrderId, $magentoOrder, $orderData);

        // Accepting the order on Glopal
        $this->acceptOrder($shortName, $apiKey, $glopalOrderId, $magentoOrder);
    }

    /**
     * Retrieve Magento order and data from cache or Glopal request.
     *
     * @param string $url
     * @param string $glopalOrderId
     * @param string $apiKey
     *
     * @return array
     * @throws Mage_Core_Exception
     */
    private function getOrderData($url, $glopalOrderId, $apiKey)
    {
        if (!isset($this->loadedOrdersData[$glopalOrderId])) {
            $this->echoToCli(
                array(
                    array('color' => '[32m', 'msg' => 'Start processing the order with id: ' . $glopalOrderId)
                )
            );
            $responseData = $this->apiHelper->makeGetHttpCall($url, $apiKey);
            $this->echoToCli(
                array(
                    array('color' => '[33m', 'msg' => 'Response:'),
                    array('color' => '[0m', 'msg' => Mage::helper('core')->jsonEncode($responseData))
                )
            );

            $orderId = $responseData['cart_transfer']['cart']['attributes']['order_id'];

            /** @var Mage_Sales_Model_Order $magentoOrder */
            $magentoOrder = Mage::getModel('sales/order')->load($orderId);
            if (empty($magentoOrder->getId())) {
                $this->echoToCli(
                    array(
                        array('color' => '[31m', 'msg' => 'No available Magento order, with id: ' . $orderId)
                    )
                );

                throw new Mage_Core_Exception('No available Magento order, with id: ' . $orderId);
            }

            $glopalOrderTotal = $responseData['summary']['total']['total'];

            return array('magento_order' => $magentoOrder, 'glopal_order_total' => $glopalOrderTotal);
        } else {
            return $this->loadedOrdersData[$glopalOrderId];
        }
    }

    /**
     * Accept order at Glopal.
     *
     * @param string $shortName
     * @param string $apiKey
     * @param string $glopalOrderId
     * @param Mage_Sales_Model_Order $magentoOrder
     *
     * @return void
     * @throws Exception
     */
    private function acceptOrder($shortName, $apiKey, $glopalOrderId, $magentoOrder)
    {
        $this->echoToCli(array(array('color' => '[32m', 'msg' => 'Start accept the order')));

        $responseData = $this->apiHelper->makePostHttpCall(
            sprintf($this->urlApiFacade . self::ACCEPT_ORDER_URL_PATTERN, $shortName, $glopalOrderId),
            $apiKey,
            array(
                'external' => array(
                    'order_id' => $magentoOrder->getId(),
                    'order_number' => $magentoOrder->getIncrementId()
                )
            )
        );

        $this->echoToCli(
            array(
                array('color' => '[33m', 'msg' => 'Accept Response:'),
                array('color' => '[0m', 'msg' => Mage::helper('core')->jsonEncode($responseData))
            )
        );
    }

    /**
     * Saves the Magento order data and refresh the memory cache.
     *
     * @param string $glopalOrderId
     * @param Mage_Sales_Model_Order $magentoOrder
     * @param array $orderData
     *
     * @return void
     * @throws Exception
     */
    private function saveOrderData($glopalOrderId, $magentoOrder, $orderData)
    {
        $orderData['magento_order'] = $magentoOrder->save();
        $this->loadedOrdersData[$glopalOrderId] = $orderData;
    }

    /**
     * Echo messages, with different colors, to cli
     *
     * @param array $data
     * @return void
     */
    private function echoToCli($data = array())
    {
        if (!count($data)) {
            return;
        }

        $result = PHP_EOL;
        foreach ($data as $datum) {
            $result .= "\033" . $datum['color'] . $datum['msg'] . " ";
        }

        echo $result . PHP_EOL;
    }
}
