<?php

/**
 * Observer
 *
 * @category   Glopal
 * @package    Glopal_Connector
 * @author     Glopal Team <support@glopal.com>
 * @license    http://opensource.org/licenses/osl-3.0.php
 */
class Glopal_Connector_Model_Observer
{
    /**
     * Fix "404 Error Page Not Found" after installing a new extension
     */
    public function hookIntoAdminhtmlInitSystemConfig($observer)
    {
        try {
            if ($session = Mage::getSingleton('admin/session')) {
                $session->setAcl(Mage::getResourceModel('admin/acl')->loadAcl());
            }
        } catch (Exception $e) {}

        return $observer;
    }

    /**
     * Display global notifications for admin
     */
    public function hookIntoGlopalConnectorNotificationsBefore($observer)
    {
        $currentUrl = Mage::helper('core/url')->getCurrentUrl();

        if (
            Mage::helper('glopal_connector')->isEnabled() &&
            Mage::helper('glopal_connector')->isGlobalNotificationsEnabled() &&
            Mage::getSingleton('admin/session')->isAllowed('system/config') &&
            Mage::helper('glopal_connector')->isInstallationMode() &&
            strpos($currentUrl, 'section/glopal_connector') === false
        ) {
            $url = Mage::getModel('adminhtml/url')->getUrl('/system_config/edit/section/glopal_connector');
            $notifications = Mage::getSingleton('glopal_connector/notification');
            $notifications->addMessage(Mage::helper('glopal_connector')->__("To continue installing Glopal, <a href=\"%s\">click here</a>", $url));
        }

        return $observer;
    }

    /**
     * Notify about new store view
     */
    public function hookIntoStoreAdd($observer)
    {
        Mage::helper('glopal_connector/instalation')->notifyInstalation();

        return $observer;
    }

    /**
     * Redirect to Glopal checkout
     *
     * @param $observer
     * @return void
     * @throws Exception
     */
    public function redirectToGlopalCheckout($observer)
    {
        $useGlopalCheckout = Mage::getStoreConfigFlag('glopal_checkout/general/use_glopal_checkout');
        if (!$useGlopalCheckout) {
            return;
        }

        $redirectUrl = Mage::getUrl('checkout/cart');
        /** @var Glopal_Connector_Helper_Data $helper */
        $helper = Mage::helper('glopal_connector');

        try {
            $quote = Mage::getSingleton('checkout/cart')->getQuote();
            $payloadData = $helper->buildVersion2Data($quote);
            $payload = Mage::helper('core')->jsonEncode($payloadData);

            $redirectUrl = $this->getRedirectUrl($payload);

            if (empty($redirectUrl)) {
                Mage::getSingleton('checkout/session')->addError($this->__('Checkout Error'));
                $redirectUrl = Mage::getUrl('checkout/cart');
            }
        } catch (Mage_Core_Exception $e) {
            Mage::logException($e);
            Mage::getSingleton('checkout/session')->addError($this->__('Checkout Error'));
        }

        Mage::app()->getResponse()->setRedirect($redirectUrl);
    }

    /**
     * Clear session after success Glopal Payment.
     *
     * @param $observer
     * @return void
     */
    public function clearSessionAfterSuccessPayment($observer)
    {
        /** @var Mage_Checkout_Model_Session $session */
        $session = Mage::getSingleton('checkout/session');
        $quoteId = $session->getGlopalQuoteId(true);

        if ($quoteId) {
            $quote = Mage::getModel('sales/quote')->load($quoteId);
            $quote->setIsActive(0)->save();

            $session->unsGlopalQuoteId();
            $session->clearHelperData();
            $session->unsetAll();
        }
    }

    /**
     * Return url for request cart transfer.
     *
     * @return string
     * @throws Exception
     */
    private function getRedirectUrl($payload)
    {
        $storeShortName = Mage::getStoreConfig('webinterpret_connector/short_name');
        $storeCode = Mage::getStoreConfig('glopal_checkout/general/store_code');
        $storeCode = str_replace('_', '-', $storeCode);

        $urlPattern = Mage::getStoreConfig('webinterpret_connector/request_url_pattern');
        $requestUrl = sprintf($urlPattern, $storeShortName, $storeCode);

        /** @var Glopal_Connector_Helper_Data $helper */
        $helper = Mage::helper('glopal_connector');

        $client = new Varien_Http_Client();
        $client->setUri($requestUrl)
            ->setConfig(array('timeout' => 15))
            ->setMethod(Zend_Http_Client::POST)
            ->setParameterGet($helper->getCartTransferGetParams())
            ->setRawData($payload)
            ->setHeaders('Content-Type: application/json')
            ->setHeaders('Accept: application/json');

        try {
            $response = $client->request();
        } catch (Exception $e) {
            Mage::logException($e);

            return '';
        }

        $responseBody = $response->getBody();
        $responseData = Mage::helper('core')->jsonDecode($responseBody);

        if ($responseData['status'] !== 'ok') {
            $errorsData = [
                'message' => 'Bad response from cart transfer',
                'response_code' => isset($responseData['code']) ? $responseData['code'] : '',
                'response_context' => isset($responseData['context']) ? $responseData['context'] : ''
            ];
            Mage::log(implode('; ', $errorsData), Zend_Log::DEBUG);

            return '';
        }

        return isset($responseData['cart_url']) ? $responseData['cart_url'] : '';
    }
}
