<?php

/**
 * @category   Glopal
 * @package    Glopal_Connector
 * @author     Glopal Team <support@glopal.com>
 * @license    http://opensource.org/licenses/osl-3.0.php
 */
class Glopal_Connector_CheckoutController extends Mage_Core_Controller_Front_Action
{
    /**
     * Handle cross-domain checkout redirect requests
     *
     * The process works like this:
     *
     * Step 1. On domain A, make a POST request to this API to create a checkout
     * redirect URL and then extract the "redirect_url" from the JSON response.
     *
     *   Example: POST /webinterpret_connector/checkout/redirect
     *
     * Step 2. On domain B, redirect the visitor to the URL. When this API
     * receives the GET request then it will create a new session for the user
     * and copy the items from the original cart.
     *
     *   Example: GET /webinterpret_connector/checkout/redirect?token=xxxxxxx
     */
    public function redirectAction()
    {
        if (!function_exists('openssl_cipher_iv_length')) {
            $this->dieWithJsonResponse(
                500,
                'Internal Server Error',
                array('error_message' => 'Newer version of OpenSSL is required.')
            );
        }
        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            return $this->createRedirect();
        }
        if ($_SERVER['REQUEST_METHOD'] === 'GET') {
            return $this->followRedirect();
        }
        $this->dieWithResponse(400, 'Bad Request');
    }

    /**
     * Return a checkout redirect URL in a JSON response
     */
    private function createRedirect()
    {
        $quoteId = Mage::getSingleton('checkout/session')->getQuote()->getEntityId();
        $redirectUrl = Mage::helper('core/url')->addRequestParam(
            Mage::helper('core/url')->getCurrentUrl(),
            array(
                'token' => $this->createRedirectToken($quoteId)
            )
        );
        $this->dieWithJsonResponse(200, 'OK', array(
            'redirect_url' => $redirectUrl
        ));
    }

    /**
     * Build redirect token by encrypting the quote id
     */
    private function createRedirectToken($quoteId)
    {
        $key = base64_decode(Mage::helper('glopal_connector')->getSecretKey());
        $iv = openssl_random_pseudo_bytes(openssl_cipher_iv_length('AES-256-CBC'));
        $token = base64_encode(json_encode(array(
            0 => $this->encryptData($quoteId, $key, $iv),
            1 => base64_encode($iv),
        )));
        return $token;
    }

    /**
     * Parse redirect token and return quote id
     *
     * @throws Exception
     */
    private function parseRedirectToken($token)
    {
        $token = json_decode(base64_decode($token));
        if (json_last_error() !== JSON_ERROR_NONE) {
            throw new \Exception('Invalid token (json error).');
        }
        if (!is_array($token) || count($token) < 2) {
            throw new \Exception('Invalid token (bad data).');
        }
        $encryptedData = $token[0];
        $iv = base64_decode($token[1]);
        $key = base64_decode(Mage::helper('glopal_connector')->getSecretKey());
        $quoteId = $this->decryptData($encryptedData, $key, $iv);
        if (!is_numeric($quoteId)) {
            throw new \Exception('Invalid quote id.');
        }
        return $quoteId;
    }

    /**
     * Encrypt data using AES-256-CBC
     */
    private function encryptData($data, $key, $iv)
    {
        return openssl_encrypt($data, 'AES-256-CBC', $key, 0, $iv);
    }

    /**
     * Decrypt data using AES-256-CBC
     */
    private function decryptData($data, $key, $iv)
    {
        return openssl_decrypt($data, 'AES-256-CBC', $key, 0, $iv);
    }

    /**
     * Copy items to the cart from an existing quote and then
     * redirect the user to the checkout.
     */
    private function followRedirect()
    {
        try {
            // Get token from URL parameter and parse it
            if (!isset($_GET['token'])) {
                throw new \Exception('Missing token.');
            }
            $quoteId = $this->parseRedirectToken($_GET['token']);

            // Load quote from database
            $quote = Mage::getSingleton('sales/quote')->load($quoteId);
            if (!is_numeric($quote->getEntityId())) {
                throw new \Exception('Failed to load quote.');
            }

            // Copy items from quote to new cart
            $cart = Mage::getSingleton('checkout/cart');
            $cart->truncate();
            foreach ($quote->getAllVisibleItems() as $quoteItem) {
                $product = Mage::getModel('catalog/product')
                    ->setStoreId(Mage::app()->getStore()->getId())
                    ->load($quoteItem->getProductId());
                $cart->addProduct($product, $quoteItem->getBuyRequest());
            }
            $cart->save();

            // Redirect to checkout
            $url = Mage::helper('glopal_connector')->getCheckoutUrl();

        } catch (\Exception $e) {
            // Something went wrong. We don't want to display the error
            // to the end user so let's redirect to the cart and place
            // the error message in an header to assist with troubleshooting.
            header('X-Glopal-Error: ' . $e->getMessage());
            $url = Mage::helper('checkout/url')->getCartUrl();
        }
        header('Location: ' . $url);
        die();
    }

    /**
     * Output JSON response and stop script execution
     *
     * @param string $statusCode
     * @param string $statusText
     * @param mixed $data
     * @return void
     */
    private function dieWithJsonResponse($statusCode, $statusText, $data)
    {
        $this->dieWithResponse($statusCode, $statusText, 'application/json', json_encode($data));
    }

    /**
     * Output response and stop script execution
     */
    private function dieWithResponse($statusCode, $statusText, $contentType = 'text/html', $data = null)
    {
        $protocol = (isset($_SERVER['SERVER_PROTOCOL']) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0');
        header($protocol . ' ' . $statusCode . ' ' . $statusText);
        if (!empty($data)) {
            header("Content-Type: $contentType");
            echo $data;
        }
        die();
    }
}
