<?php

require_once('AbstractController.php');
require_once('InvoiceController.php');

class Glopal_Connector_CreditmemoController extends Glopal_Connector_AbstractController
{
    const CREDITMEMO_COMMENT_CREATED_BY_GLOPAL = '--CREDITMEMO WAS CREATED BY GLOPAL API--';

    /**
     * @return array
     * @throws InvalidArgumentException if we can't find the order in the database or the order_id param was empty
     */
    protected function listCreditmemos()
    {
        $orderId = $this->getRequest()->getParam('order_id', null);
        if (empty($orderId)) {
            throw new InvalidArgumentException('Order Id was not provided');
        }
        $order = Mage::getModel('sales/order')->load($orderId);
        if (null == $order->getId()) {
             throw new InvalidArgumentException('Wrong order Id was provided');
        }
        if (!$order->hasCreditmemos()) {
            return [];
        }

        $creditmemos = array_values($order->getCreditmemosCollection()->getItems());
        return array_map(function($creditmemo){
            $comments =  $creditmemo->getCommentsCollection()->getItems();
            $createdByGlopal = array_filter($comments, function($comment) {
                return $comment->getComment() == Glopal_Connector_CreditmemoController::CREDITMEMO_COMMENT_CREATED_BY_GLOPAL;
            });
            return [
                'creditmemo_id'     => $creditmemo->getId(),
                'grand_total'       => $creditmemo->getGrandTotal(),
                'increment_id'      => $creditmemo->getIncrementId(),
                'createdByGlopal' => (bool) count($createdByGlopal)
            ];
        }, $creditmemos);
    }

    /**
     * @return array
     * @throws InvalidArgumentException if we can't find the order in the database or the order_id param was empty
     * @throws OutOfRangeException in case the order does not have proper invoices to be refunded
     */
    protected function createCreditMemo() 
    {
        $orderId = $this->getRequest()->getParam('order_id', null);
        if (empty($orderId)) {
            throw new InvalidArgumentException('Order Id was not provided');
        }
        $order = Mage::getModel('sales/order')->load($orderId);
        if (null == $order->getId()) {
             throw new InvalidArgumentException('Wrong order Id was provided');
        }
        if (!$order->hasInvoices()) {
            return [
            ];
        }
        $invoices = $order->getInvoiceCollection()->getItems();
        $invoicesCreatedByGlopal = array_filter($invoices, function($invoice) {
            if (!$invoice->canRefund()) {
                return false;
            }
            $comments =  $invoice->getCommentsCollection()->getItems();
            $isCreatedByGlopal = array_filter($comments, function($comment) {
                return $comment->getComment() == Glopal_Connector_InvoiceController::INVOICE_COMMENT_CREATED_BY_GLOPAL;
            });
            return count($isCreatedByGlopal) > 0;
        });
        if (count($invoices) > 1  || count($invoicesCreatedByGlopal) != 1) {
            throw new OutOfRangeException(
                sprintf(
                    'Fround a total of: %d, from which a number of: %d were created by Glopal Api and can be refunded',
                    count($invoices),
                    count($invoicesCreatedByGlopal)
                )
            );
        }

        $service = Mage::getModel('sales/service_order', $order);
        $invoiceToRefund = array_shift($invoicesCreatedByGlopal);

        $creditmemo = $service->prepareInvoiceCreditmemo($invoiceToRefund);
        $creditmemo->refund();
        $creditmemo->addComment(
            self::CREDITMEMO_COMMENT_CREATED_BY_GLOPAL
        );

        Mage::getModel('core/resource_transaction')
        ->addObject($creditmemo)
        ->addObject($creditmemo->getOrder())
        ->addObject($creditmemo->getInvoice())
        ->save();
        
        return [
            'creditmemo_id'     => $creditmemo->getId(),
            'grand_total'       => $creditmemo->getGrandTotal(),
            'increment_id'      => $creditmemo->getIncrementId(),
            'createdByGlopal'   => true
        ];
    }

    /**
     * @return array
     * @throws InvalidArgumentException if a different request method type is used
     */
    protected function process()
    {
        if ($this->getRequest()->isGet()) {
            return $this->listCreditmemos();
        }
        if ($this->getRequest()->isPost()) {
            return $this->createCreditMemo();
        }
        throw new InvalidArgumentException('Wrong method used');
    }
}
