<?php

class Glopal_Connector_PaymentController extends Mage_Core_Controller_Front_Action
{
    /**
     * Redirect to payment Glopal page after place an order.
     */
    public function redirectAction()
    {
        /** @var Mage_Checkout_Model_Session $session */
        $session = Mage::getSingleton('checkout/session');
        $orderId = $session->getLastOrderId();
        /** @var Mage_Sales_Model_Order $order */
        $order = Mage::getModel('sales/order')->load($orderId);

        if (null == $order->getId()) {
            throw new InvalidArgumentException("Order with given ID: $orderId does not exists");
        }

        try {
            $redirectData = $this->prepareRedirectUrl($order);
        } catch (Exception $e) {
            Mage::logException($e);
            $this->errorResponse($e->getMessage(), 'checkout/cart');

            return;
        }

        $quoteId = $order->getQuoteId();
        $session->setGlopalQuoteId($quoteId);
        $this->restoreQuote($quoteId);

        $this->getResponse()
            ->setRedirect($redirectData['cart_url'])
            ->sendResponse();

        return;
    }

    /**
     * Glopal payment success callback action.
     */
    public function successAction()
    {
        /** @var Mage_Core_Controller_Request_Http $request */
        $request = Mage::app()->getRequest();
        $storeId = Mage::app()->getStore()->getStoreId();
        $secret = Mage::getStoreConfig('webinterpret_connector/key', $storeId);

        if (empty($secret)) {
            $this->errorResponse('Secret is not set', 'checkout/cart');

            return;
        }

        $currentUrl = $this->getCurrentUrlFromRequest($request);
        if (!$this->verifyHmacSignature($currentUrl, $secret)) {
            $this->errorResponse('Signature is not valid', 'checkout/cart');

            return;
        }

        $magentoOrderId = $request->getParam('oid');
        /** @var Mage_Sales_Model_Order $order */
        $order = Mage::getModel('sales/order')->load($magentoOrderId);
        if (null === $order->getId()) {
            $this->errorResponse(
                "Order with given id $magentoOrderId not found",
                'checkout/cart'
            );

            return;
        }

        /** @var Mage_Checkout_Model_Session $session */
        $session = Mage::getSingleton('checkout/session');
        $session->setLastSuccessQuoteId($order->getQuoteId());
        $session->setLastQuoteId($order->getQuoteId());
        $session->setLastOrderId($order->getId());

        // Redirect customer to success page.
        $this->_redirect('checkout/onepage/success');
    }

    /**
     * Retrieve current url from request.
     *
     * @return string
     */
    public function getCurrentUrlFromRequest(Mage_Core_Controller_Request_Http $request)
    {
        return $request->getScheme() . '://' . $request->getHttpHost() . $request->getRequestUri();
    }

    /**
     * Prepare order redirect url.
     *
     * @param Mage_Sales_Model_Order $order
     *
     * @return array
     * @throws Mage_Core_Model_Store_Exception
     * @throws Zend_Http_Client_Exception
     */
    private function prepareRedirectUrl($order)
    {
        $storeId = Mage::app()->getStore()->getStoreId();
        $destination = $order->getShippingAddress()->getCountryId();

        /** @var Glopal_Connector_Helper_Data $dataHelper */
        $dataHelper = Mage::helper('glopal_connector');

        $hsCodeAttributeCode = Mage::getStoreConfig(
            'glopal_cart_transfer/product_data/magento_attibute_hs_code',
            $storeId
        );
        if(empty($hsCodeAttributeCode)) {
            $hsCodeAttributeCode = 'hs_code';
        }

        $cooCodeAttributeCode = Mage::getStoreConfig(
            'glopal_cart_transfer/product_data/magento_attibute_coo',
            $storeId
        );
        if(empty($cooCodeAttributeCode)) {
            $cooCodeAttributeCode = 'country_of_manufacture';
        }

        $configurableProducts = array();
        /** @var Mage_Sales_Model_Order_Item $item */
        foreach ($order->getItemsCollection() as $item) {
            if (!$item->getParentItemId() && $item->getProductType() === 'configurable') {
                $configurableProducts[$item->getId()] = $item;
            }
        }

        $items = array();
        foreach ($order->getItemsCollection() as $orderItem) {
            if (!$orderItem->getParentItemId() && $orderItem->getProductType() === 'configurable') {
                continue;
            }

            $item = array();
            $product = null;

            if ($orderItem->getParentItemId() && isset($configurableProducts[$orderItem->getParentItemId()])) {
                // product + variant
                $configurableItem = $configurableProducts[$orderItem->getParentItemId()];
                $product = $configurableItem->getProduct();

                $orderOptions = $configurableItem->getProductOptions()['attributes_info'];
                $itemOptions = [];
                foreach ($orderOptions as $orderOption) {
                    $itemOptions[] = array(
                        // TODO: collect option_id, value_id
                        'name' => $orderOption['label'],
                        'value' => $orderOption['value'],
                    );
                }

                // We are using "original_price" for Louboutin (instead "price")
                $item = array_merge($item, array(
                    'regular_price' => (float) $configurableItem->getOriginalPrice(),
                    'variant_id' => $configurableItem->getProductId(),
                    'options' => $itemOptions
                ));
            } else {
                // product only
                $product = $orderItem->getProduct();
                // We are using "original_price" for Louboutin (instead "price")
                $item = array_merge($item, array('regular_price' => (float) $orderItem->getOriginalPrice()));
            }

            $productId = $product->getId();
            $item = array_merge($item, array(
                'id' => $productId,
                'name' => $product->getName(),
                'sku' => $orderItem->getSku(),
                'url' => $orderItem->getProduct()->getUrlInStore(),
                'quantity' => (int) $orderItem->getQtyOrdered(),
                'image_url' => $this->getProductImageUrl($product),
                'hs_code' => $dataHelper->getProductAttribute(
                    $productId,
                    $storeId,
                    $hsCodeAttributeCode
                ),
                'country_id_of_origin' => $dataHelper->getProductAttribute(
                    $productId,
                    $storeId,
                    $cooCodeAttributeCode
                ),
            ));

            $items[] = $item;
        }

        $hideSteps = array('display' => 'hidden');
        $customerInfo = array(
            'email' => $order->getCustomerEmail(),
            'first_name' => $order->getCustomerFirstname(),
            'last_name' => $order->getCustomerLastname()
        );

        $billingAddress = $this->prepareAddress($order->getBillingAddress());
        $shippingAddress = $this->prepareAddress($order->getShippingAddress());

        $postParameters = array(
            'store_id' => Mage::getStoreConfig('webinterpret_connector/short_name', $storeId),
            'cart' => array(
                'currency' => $order->getOrderCurrencyCode(),
                'prices' => array(
                    'model' => "fixed",
                    'include_destination_duties' => true,
                    'include_destination_tax' => true
                ),
                'external_id' => $order->getQuoteId(),
                'attributes' => array(
                    'store_view_id' => $storeId,
                    'order_id' => $order->getId()
                )
            ),
            'visitor' => array(
                'currency' => $order->getOrderCurrencyCode(),
                // $hideSteps - For direct redirect to payment step
                'customer' => array_merge($customerInfo, $hideSteps),
                'billing_address' => array_merge($billingAddress, $hideSteps),
                'shipping_address' => array_merge($shippingAddress, $hideSteps),
                'destination' => $destination
            ),
            'checkout' => array(
                'success_url' => Mage::getUrl(
                    'glopal_connector/payment/success',
                    array(
                        '_query' => array('oid' => $order->getId())
                    )
                ),
                'success_url_enable_hmac' => true
            ),
            'items' => $items,
            'shipping' => array(
                'quote' => array(
                    array(
                        'display_name' => $order->getShippingDescription(),
                        'price' => (float) $order->getShippingAmount(),
                        'service_name' => 'EXPRESS',
                    )
                ),
                'selected_shipping_service' => 'express',
                'display' => 'hidden' // For direct redirect to payment step
            ),
            'referrer' => Mage::getUrl('checkout/cart')
        );

        $discountAmount = -1 * $order->getDiscountAmount();
        if ($discountAmount) {
            $postParameters['discounts'][] = array(
                'name' => 'fixed_amount',
                'amount' => $discountAmount
            );
        }

        $tracking = $this->getTrackingParams($dataHelper);
        if (!empty($tracking)) {
            $postParameters['tracking'] = $tracking;
        }

        /** @var Glopal_Connector_Helper_Api $paymentHelper */
        $paymentHelper = Mage::helper('glopal_connector/api');
        $token = Mage::getStoreConfig('glopal_checkout/general/glopal_api_key', $storeId);

        return $paymentHelper->makePostHttpCall(
            Mage::getStoreConfig('webinterpret_connector/url_cart_transfer'),
            $token,
            $postParameters
        );
    }

    /**
     * Gets correct product image url.
     *
     * @param Mage_Catalog_Model_Product $product
     *
     * @return string
     */
    private function getProductImageUrl($product)
    {
        /** @var Mage_Catalog_Helper_Image $helper */
        $helper = Mage::helper('catalog/image');

        return (string) $helper->init($product, 'image');
    }

    /**
     * Prepares address in request format.
     *
     * @param Mage_Sales_Model_Order_Address $address
     * @return array
     */
    private function prepareAddress(Mage_Sales_Model_Order_Address $address)
    {
        return array(
            'address1' => $address->getStreetFull(),
            'company' => null !== $address->getCompany() ? $address->getCompany() : '',
            'first_name' => $address->getFirstname(),
            'last_name' => $address->getLastname(),
            'postcode' => $address->getPostcode(),
            'city' => $address->getCity(),
            'state' => null !== $address->getRegionCode() ? $address->getRegionCode() : '',
            'country_code' => $address->getCountryId(),
            'telephone' => $address->getTelephone()
        );
    }

    /**
     * Returns tracking params in appropriate format.
     *
     * @param Glopal_Connector_Helper_Data $helper
     *
     * @return array
     */
    private function getTrackingParams($helper)
    {
        $result = array();
        $analyticsParams = $helper->getCartTransferAnalyticsParams();

        if (isset($analyticsParams['cookies'])) {
            $result['cookies'] = $analyticsParams['cookies'];
        }

        if (isset($analyticsParams['ga'])) {
            $result['client_ids']['ga'] = $analyticsParams['ga'];
        }

        if (isset($analyticsParams['ga4'])) {
            $result['client_ids']['ga4_session_id'] = $analyticsParams['ga4'];
        }

        return $result;
    }

    /**
     * Verify HMAC signature by given request url.
     *
     * @param string $url
     * @param string $secret
     */
    private function verifyHmacSignature($url, $secret)
    {
        // Check if HMAC signature verification enabled
        if (!Mage::getStoreConfigFlag('glopal_checkout/general/is_hmac_signature_enabled')) {
            return true;
        }

        // extract expected "sig" and sort query params
        $parsedUrl = parse_url($url);
        parse_str($parsedUrl['query'], $queryParams);
        $sig = $queryParams['sig'];
        unset($queryParams['sig']);
        ksort($queryParams);

        $expectedSig = @hex2bin($sig);
        if ($expectedSig === false) {
            return false;
        }

        // calculate over url with sorted params
        $urlToSign = sprintf('%s://%s%s', $parsedUrl['scheme'], $parsedUrl['host'], $parsedUrl['path']);
        if (!empty($queryParams)) {
            $urlToSign .= '?' . http_build_query($queryParams);
        }

        $calculatedSig = hash_hmac('sha256', $urlToSign, $secret, true);

        // verify
        return hash_equals($calculatedSig, $expectedSig);
    }

    /**
     * Log error message and redirect customer to appropriate url.
     *
     * @param string $message
     * @param string $redirectRoute
     * @return void
     */
    private function errorResponse($message, $redirectRoute)
    {
        Mage::log($message, Zend_Log::ERR);
        $this->_redirect($redirectRoute);
    }

    /**
     * Restore last active quote based on checkout session
     *
     * @param int|string $quoteId
     * @return bool True if quote restored successfully, false otherwise
     */
    private function restoreQuote($quoteId)
    {
        $quote = Mage::getModel('sales/quote')->load($quoteId);
        if (null !== $quote && $quote->getId()) {
            $quote->setIsActive(1)
                ->setReservedOrderId(null)
                ->save();

            /** @var Mage_Checkout_Model_Session $checkoutSession */
            $checkoutSession = Mage::getSingleton('checkout/session');
            $checkoutSession->replaceQuote($quote)->unsLastRealOrderId();

            return true;
        }

        return false;
    }
}