<?php

/**
 * @category   Glopal
 * @package    Glopal_Connector
 * @author     Glopal Team <support@glopal.com>
 * @license    http://opensource.org/licenses/osl-3.0.php
 */

class Glopal_Connector_QuoteRequest
{
    protected $data = null;

    public function setData($data)
    {
        $this->data = $data;
    }

    public function hasShippingAddress()
    {
        return $this->data->checkout->shipping_address;
    }

    public function getStoreViewId()
    {
        return $this->data->store->store_view_id;
    }

    public function getShippingCountryCode()
    {
        return $this->data->checkout->shipping_address->country_code;
    }

    public function getShippingPostcode()
    {
        return $this->data->checkout->shipping_address->postal_code;
    }

    public function getShippingCompany()
    {
        return $this->data->checkout->shipping_address->contact->company_name;
    }

    public function getShippingFirstname()
    {
        return $this->data->checkout->shipping_address->contact->first_name;
    }

    public function getShippingLastname()
    {
        return $this->data->checkout->shipping_address->contact->last_name;
    }

    public function getShippingVatId()
    {
        return $this->data->checkout->shipping_address->contact->tax_number;
    }

    public function getShippingCity()
    {
        return $this->data->checkout->shipping_address->city;
    }

    public function getShippingRegion()
    {
        return $this->data->checkout->shipping_address->state;
    }

    public function getShippingRegionCode()
    {
        return $this->data->checkout->shipping_address->state_code;
    }

    public function getShippingStreet()
    {
        $street = [];
        if($this->data->checkout->shipping_address->street1) {
            $street[]= $this->data->checkout->shipping_address->street1;
        }
        if($this->data->checkout->shipping_address->street2) {
            $street[]= $this->data->checkout->shipping_address->street2;
        }
        return $street;
    }

    public function getShippingTelephone()
    {
        return $this->data->checkout->shipping_address->phone;
    }

    public function getShippingCountryId()
    {
        return $this->data->checkout->shipping_address->country_code;
    }

    public function getShippingEmail()
    {
        return $this->data->checkout->shipping_address->email;
    }

    public function getItems()
    {
        return $this->data->items;
    }

    public function getCouponCode()
    {
        return $this->data->checkout->discount_code;
    }
}

class Glopal_Connector_QuoteRequestValidator
{
    /**
     * Validate quote request object
     *
     * @todo: implement this function
     */
    public function validate($request)
    {
        $requiredState = explode(',',Mage::getStoreConfig('general/region/state_required', $request->getStoreViewId()));
        $errors = [];
        if(in_array($request->getShippingCountryCode(), $requiredState) && (strlen($request->getShippingRegion()) == 0) && (strlen($request->getShippingRegionCode()) == 0)) {
            $errors[]= __('State or state code is required');
        }
        return $errors;
    }
}

class Glopal_Connector_QuoteController extends Mage_Core_Controller_Front_Action
{
    private $_regionCollection = null;

    public function getRegionCollection($countryId)
    {
        if (!$this->_regionCollection) {
            $this->_regionCollection = Mage::getModel('directory/region')->getResourceCollection()
                ->addCountryFilter($countryId)
                ->setOrder('region_id', Varien_Data_Collection::SORT_ORDER_ASC)
                ->load();
        }
        return $this->_regionCollection;
    }
    /**
     * Create new quote in memory and return shipping rates
     */
    function indexAction()
    {
        // Parse and validate input
        $request = new Glopal_Connector_QuoteRequest();
        $validator = new Glopal_Connector_QuoteRequestValidator();
        $request->setData(json_decode($this->getRequest()->getRawBody()));
        $errors = $validator->validate($request);
        if (count($errors) > 0) {
            header('Content-Type: application/json');
            echo json_encode([
              'status' => 'error',
              'errors' => $errors
            ]);
            die();
        }

        $response = $warnings = [];

        try {
            // Create quote in memory
            $quote = Mage::getModel('sales/quote')->setStoreId($request->getStoreViewId());
            $quote->setCurrency($quote->getStore()->getCurrentCurrencyCode());

            // Add products to quote
            foreach ($request->getItems() as $item) {
                $product = Mage::getModel('catalog/product')->load($item->id);
                if(!$product || !$product->getId()) {
                    throw new \Exception(__('Product %s does not exist', $item->id));
                }
                $product->getStockItem()->setUseConfigManageStock(false);
                $product->getStockItem()->setManageStock(false);
                $params = array('qty'=>$item->quantity);
                if($item->variant_id) {
                    if($product->getTypeId() != 'configurable') {
                        throw new \Exception('Variants are not supported');
                    }
                    $productAttributeOptions = $product->getTypeInstance(true)->getConfigurableAttributesAsArray($product);
                    $variant = Mage::getModel('catalog/product')->load($item->variant_id);
                    if(!$variant || !$variant->getId()) {
                        throw new \Exception(__('Variant %s does not exist', $item->variant_id));
                    }
                    $superAttribute = array();
                    foreach ($productAttributeOptions as $productAttribute) {
                        $allValues = array_column($productAttribute['values'], 'value_index');
                        $currentProductValue = $variant->getData($productAttribute['attribute_code']);
                        if (in_array($currentProductValue, $allValues)) {
                            $superAttribute[$productAttribute['attribute_id']] = $currentProductValue;
                            if($item->options) {
                                foreach($item->options as $key => $option) {
                                    if($option->option_id == $productAttribute['attribute_id']) {
                                        unset($item->options[$key]);
                                    }
                                }
                            }
                        }
                    }
                    $params['super_attribute'] = $superAttribute;
                }
                if($item->options) {
                    $options = array();
                    foreach($item->options as $option) {
                        $options[$option->option_id] = $option->value_id ? $option->value_id : $option->value;
                    }
                    $params['options'] = $options;
                }
                $paramsObj = new Varien_Object($params);
                $quote->addProduct($product, $paramsObj);
            }

            $regionId = null;
            $regions = $this->getRegionCollection($request->getShippingCountryCode());
            foreach($regions as $region) {
                if($request->getShippingRegionCode() == $region->getCode() || $request->getShippingRegion() == $region->getDefaultName()) {
                    $regionId = $region->getRegionId();
                }
            }

            // Create shipping address
            if($request->hasShippingAddress()) {
                $quote->getShippingAddress()
                    ->setCountryId($request->getShippingCountryCode())
                    ->setPostcode($request->getShippingPostcode())
                    ->setEmail($request->getShippingEmail())
                    ->setFirstname($request->getShippingFirstname())
                    ->setLastname($request->getShippingLastname())
                    ->setCompany($request->getShippingCompany())
                    ->setVatId($request->getShippingVatId())
                    ->setCity($request->getShippingCity())
                    ->setStreet($request->getShippingStreet())
                    ->setRegion($request->getShippingRegion())
                    ->setRegionId($regionId)
                    ->setTelephone($request->getShippingTelephone())
                ;
            }
            if($coupon = $request->getCouponCode()) {
                $quote->setCouponCode($coupon);
            }
            $quote->getShippingAddress()->collectTotals();
            $quote->getShippingAddress()->setCollectShippingRates(true);
            $quote->getShippingAddress()->collectShippingRates();
            $quote->collectTotals();
            $rates = $quote->getShippingAddress()->getShippingRatesCollection();
            if($coupon = $request->getCouponCode()) {
                $oCoupon = Mage::getModel('salesrule/coupon')->load($coupon, 'code');
                if($oCoupon->getRuleId()) {
                    $oRule = Mage::getModel('salesrule/rule')->load($oCoupon->getRuleId());
                    if($oRule->getRuleId()) {
                        $appliedRules = explode(",",$quote->getAppliedRuleIds());
                        if(!in_array($oRule->getRuleId(), $appliedRules)) {
                            $quote->setCouponCode('');
                            $warnings[]= __('Coupon code is not applicable');
                        }
                    } else {
                        $quote->setCouponCode('');
                        $warnings[]= __('Coupon code is not valid');
                    }
                } else {
                    $quote->setCouponCode('');
                    $warnings[]= __('Coupon code is not valid');
                }
            }

            $response = Mage::helper('glopal_connector')->buildVersion2Data($quote, false);
            
            if(!empty($warnings)) {
                $response['warnings'] = $warnings;
            }

            // Build response
            if($request->hasShippingAddress()) {
                $response['checkout'] = array(
                    'shipping_address' => $quote->getShippingAddress()->getData(),
                );
            }
            $response['shipping_options'] = array();
            foreach ($rates as $rate) {

                $shippingPrice = Mage::helper('tax')->getShippingPrice($rate->getPrice(), true, $quote->getShippingAddress(), $quote->getCustomerTaxClassId(), $quote->getStoreId());

                $response['shipping_options'][] = array(
                    'total_price' => $shippingPrice,
                    'service_code' => $rate->getCode(),
                    'service_type' => $rate->getCarrierTitle(),
                    'service_name' => $rate->getMethodTitle(),
                    'delivery_text' => $rate->getMethodDescription(),
                    'attributes' => $rate->getData()
                );
            }
        } catch(\Exception $e) {
            header('Content-Type: application/json');
            echo json_encode([
              'status' => 'error',
              'errors' => [$e->getMessage()]
            ]);
            die();
        }

        // Return JSON response
        header('Content-Type: application/json');
        echo json_encode($response);
        die();
    }
}
